#if !defined(CRTSEC_H)
#define CRTSEC_H
// listing:	3
// file:	critical.h
// author:	allan kelly

#if !defined(_WINDOWS_)
#include <windows.h>
#endif

// One critical section is used by multiple threads
class CriticalSection
{
private:
	CRITICAL_SECTION m_criticalSection;

	void EnterCriticalSection()
	{
		::EnterCriticalSection(&m_criticalSection);
	}

	void LeaveCriticalSection()
	{
		::LeaveCriticalSection(&m_criticalSection);
	}

public:
	friend class CriticalToken;

	// create and initialize Critical section
	explicit CriticalSection()
	{
		InitializeCriticalSection(&m_criticalSection);
	}

	// destroy critical section
	~CriticalSection()
	{
		DeleteCriticalSection (&m_criticalSection);
	}

private:
	// block default copies ---------------------------
	CriticalSection(const CriticalSection&);
	CriticalSection& operator=(const CriticalSection&);
};

// Critcal section token allocated on entry
class CriticalToken
{
private:
	CriticalSection &m_criticalSection;

public:
	explicit CriticalToken(CriticalSection &cs) : m_criticalSection(cs)
	{
		m_criticalSection.EnterCriticalSection();
	}

	~CriticalToken()
	{
		m_criticalSection.LeaveCriticalSection();
	}

private:
	// block default copies ---------------------------
	CriticalToken(const CriticalToken&);
	CriticalToken& operator=(const CriticalToken&);
};
#endif  // CRTSEC_H