// listing:		4
// file:		csexample.cpp		
// author:		allan kelly

#include <iostream.h>
#include "threadtemplate.h"
#include "critical.h"

class Worker
{
public:
	Worker(CriticalSection&);
	void Run();
	int Result();

private:
	CriticalSection &m_criticalSection;
	int	m_total;
};

Worker::Worker(CriticalSection& cs) : m_criticalSection(cs)
{
}

void Worker::Run()
{
	// must declare outside
	int first;
	int second;

	{	// enter critical section while we do io work
		CriticalToken critical(m_criticalSection);
		cout << "Enter two numbers..." << endl;
		cout << "First number: ";
		cin >> first;
		cout << "Second number:";
		cin >> second;
	}	// scope take us out of critical section

	//  do the work......
	m_total = first + second;
}

int Worker::Result()
{
	return m_total;
}


int main()
{
	cout << "Three threads and a critical section" << endl;

	// create a critical section
	CriticalSection cs;

	// create the worker objects
	Worker worker1(cs);
	Worker worker2(cs);

	{	// threads run in this scope
		// create threads
		ThreadTemplate<Worker> thread1;
		ThreadTemplate<Worker> thread2;

		// run threads
		// both try to use cout and cin but critcal section guards i/o
		thread1.RunThread(worker1);
		thread2.RunThread(worker2);
	}	// thread destructors will wait for threads to terminate

	// worker objects still exists (threads don't)
	cout << "Worker 1 results = " << worker1.Result() << endl;
	cout << "Worker 2 results = " << worker2.Result() << endl;
	cout << "That's all folks!" << endl;

	return 0;
}
